
from Products.Archetypes.config import TOOL_NAME
from Products.Archetypes.public import *
from BatyBase import BatyBase
from BatyObject import BatyObject
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from DateTime import DateTime
from Products.Archetypes.debug import log
from Products.ArchBaty.BatyUtils import callbase
from BatyAffaire import isAffaireStock
from Products.ArchBaty import BatyPermissions
from Products.ArchBaty.properties import BatyFields



def addBatyBaseJournal(self,id, **kwargs):
    """ajoute un nouveau journal"""
    o = BatyBaseJournal(id,**kwargs)
    
    if 'title' in kwargs.keys():
        o.setTitle(kwargs['title'])
    self._setObject(id, o)





class BatyBaseJournal(BatyBase):
    """contient les journaux analytiques"""
    portal_type = meta_type = 'BatyBaseJournal' 
    archetype_name = 'BatyBaseJournal'
    schema=BatyBase.schema
    factory_type_information={
        'allowed_content_types':('BatyJournal','BatyJournalHeure'),
        'filter_content_types':1,
        'immediate_view':'batybase_custom_view'
        }

    actions=(
        {
		'id'           : 'view',
		'name'         : 'Journal analytique',
		'action'       : 'string:batybasejournal_custom_view',
		'permissions'  : (BatyPermissions.VIEW_BATY_JOURNAL, ),
		'category'      : 'object',

                },
        )
    security=ClassSecurityInfo()

    def default_action(self):
        """ retourne l'action par defaut """
        return 'view'

registerType(BatyBaseJournal)

    
class BatyJournal(BatyBase):
    """ saisie du journal analytique """
    portal_type = meta_type = 'BatyJournal' 
    archetype_name = 'BatyJournalMatiere'   #this name appears in the 'add' box
    content_icon='folder_icon_m.png'
    schema=BatyBase.schema + Schema((
         StringField('periode',
                     
                      
                     required=1,
                     vocabulary="getPeriodeEnCours",
                     write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                     searchable=1,
                     
                     schemata='Periode',
                     widget=SelectionWidget(label_msgid='periode',
                                         size="3",
                                         maxlength="2",
                                         description_msgid='description_periode',
                                         i18n_domain='archbaty',   
                                            
                                            ),
                    ),

         DateTimeField('date_de_saisie',
                       required=1,
                       searchable=1,
                       default= DateTime(),
                       accessor="formatDate",
                       edit_accessor = "now",
                       write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                       widget=StringWidget(label_msgid='date_de_saisie',
                                             description_msgid='description_date_de_saisie',
                                             i18n_domain='archbaty',
                                             format='%d/%m/%Y',
                                             )
                       ),
        
         
         
     ))

    actions=(
        {
		'id'           : 'view',
		'name'         : 'Journal analytique',
		'action'       : 'string:batyjournal_custom_view',
		'permissions'  : (BatyPermissions.VIEW_BATY_JOURNAL, ),
		'category'      : 'object',

	},
        {
		'id'           : 'edit',
		'name'         : 'Edit',
		'action'       : 'string:base_edit',
		'permissions'  : (BatyPermissions.EDIT_BATY_JOURNAL, ),
		'category'      : 'object',

	},
        {
                'id'           : 'local_roles',
		'name'         : 'Local Roles',
		'action'       : 'string:${folder_url}/folder_localrole_form',
		'permissions'  : (CMFCorePermissions.ManageProperties, ),
		'category'      : 'folder',
        },
        {
                'id'           : 'rapport',
		'name'         : 'Rapport',
		'action'       : 'string:${folder_url}/rapport_journal/getPdf_file',
		'permissions'  : (BatyPermissions.VIEW_BATY_JOURNAL, ),
                'condition' : """python: portal.portal_workflow.getInfoFor(object,'state_mouvement',0)=='cloture'""",
		'category'      : 'object', 
        },
        )
        
        
        

    
    factory_type_information={
        'allowed_content_types':('BatyJournalLigne',),
        'filter_content_types':1,
        'immediate_view':'batybase_custom_view'
        }

    security=ClassSecurityInfo()

    def now(self):
        return DateTime().strftime('%d/%m/%Y')

    def formatDate(self):
        return getattr(self,'date_de_saisie',DateTime()).strftime('%d/%m/%Y')

    def get_last_affaire(self):
        
        return getattr(self,'last_affaire',self.getListAffaire()[0])

    def getFirstAffaire(self):

        listAffaire=self.getListAffaire()
        if len(listAffaire):
            return  getToolByName(self,"archetype_tool").getObject(listAffaire[0])

    def getFirstArticle(self):

        listArticle=self.getListArticle()
        if len(listArticle):
            return getToolByName(self,"archetype_tool").getObject(listArticle[0])
        return None

    def set_last_affaire(self,value):
        setattr(self,'last_affaire',value)

    def get_last_article(self):
        return getattr(self,'last_article',None)


    def get_current_periode(self):
        return "%02d" % (DateTime(self.CreationDate()).month(),)
        

    def getListAffaire(self):
        return self.getEnt().getListAffaire()

    def getListArticle(self):
        return self.getEnt().getListArticle()


    def getListDocument(self):
        return self.getEnt().getListDocument()

    def getPeriodeEnCours(self):
        return self.getEnt().getPeriodeEnCours()


    # def getEnt(self):
#         return self.getEntreprise()
#         if hasattr(self,'entreprise'):
#             archetype_tool = getToolByName(self, TOOL_NAME)
#             ## on recupere l'uid
#             entreprise=archetype_tool.lookupObject(uid=getattr(self,'entreprise',None))
#             return entreprise
#         else:
#             raise 


    def getBaseArticle(self):
        """retourne la base des articles"""
        return self.getEnt().getBaseArticle()

    def getBaseDocument(self):
        """ retourne la base  de documentation de l'entreprise """
        try:
            return self.getEnt()['document']
        except:
            return self

    def getListCatalogArticle(self):
        """reourne la, liste des article de l'entreprise""" 
        return self.getEnt().getListCatalogArticle()


    def folder_contents(self):
        """ return default view """
        action=self.getTypeInfo().getActionById(self.default_action())
        #log(action)
        return self.view()
    ## method

    def default_action(self):
        """ retourne l'action par defaut """
        return 'view'


    def compute_prix_total_per_document(self,uid_document):
        """ retourne la somme des prix  pour un document donne """
        
        list=self.objectItems('BatyJournalLigne')
        somme=0.0 


        for item in list:
#            log(uid_document +' == ' + item[1].getDocument())
            if getattr(item[1],'document') == uid_document:
               somme=somme + item[1].getPrix_total()
        return somme

    def compute_prix_total_per_article(self,uid_article):
        """ retourne la somme des prix d'un article """
        list=self.objectItems('BatyJournalLigne')
        somme=0.0 


        for item in list:
            #log(uid_article +' == ' + item[1].getArticle())
            if getattr(item[1],'article') == uid_article:
               somme=somme + item[1].getPrix_total()
        return somme

    def get_periode_type(self):
        """ retourne si la periode est de type pair ou impair """
        periode=self.getPeriode()
        if int(periode[0:2]) % 2 == 0:
            return 'pair'
        else:
            return 'impair'
        #periode_id=periode[0:2]


registerType(BatyJournal)



class BatyJournalHeure(BatyJournal,):
    portal_type = meta_type = 'BatyJournalHeure' 
    archetype_name = 'BatyJournalHeure'   #this name appears in the 'add' box
    content_icon='folder_icon_h.png'
    schema=BatyBase.schema + BatyJournal.schema
    actions=(

        
        
        {
		'id'           : 'content',
		'name'         : 'Journal des heures',
		'action'       : 'string:batyjournal_heure_custom_view',
		'permissions'  : (BatyPermissions.VIEW_BATY_JOURNAL, ),
		'category'      : 'object',

	},
        {
		'id'           : 'edit',
		'name'         : 'Edit',
		'action'       : 'string:base_edit',
		'permissions'  : (BatyPermissions.EDIT_BATY_JOURNAL, ),
		'category'      : 'object',

	},
        {
                'id'           : 'local_roles',
		'name'         : 'Local Roles',
		'action'       : 'string:${folder_url}/folder_localrole_form',
		'permissions'  : (CMFCorePermissions.ManageProperties, ),
		'category'      : 'folder',
        },
	{
                'id'           : 'view',
		'name'         : 'Saisie',
		'action'       : 'string:${folder_url}/saisie_journal_heure',
		'permissions'  : (BatyPermissions.EDIT_BATY_JOURNAL, ),
                'condition' : """python: portal.portal_workflow.getInfoFor(object,'state_mouvement',0)!='cloture'""",
		'category'      : 'object', 
        },
        {
                'id'           : 'rapport',
		'name'         : 'Rapport',
		'action'       : 'string:${folder_url}/rapport_journal/getPdf_file',
		'permissions'  : (BatyPermissions.VIEW_BATY_JOURNAL, ),
                'condition' : """python: portal.portal_workflow.getInfoFor(object,'state_mouvement',0)=='cloture'""",
		'category'      : 'object', 
        },
        )
 
    factory_type_information={
        'allowed_content_types':('BatyJournalHeureLigne',),
        'filter_content_types':1,
        'immediate_view':'batybase_custom_view',
        }

    security=ClassSecurityInfo()

    def getAllowedTypes(self):
        """ pour eviter d'avoir des problemes d'ajout intenpestif """
        return []
        
    def default_heure(self):
        """ retourne le nombre que fait le premier salarie dans la liste """
        liste_salarie= self.getList(type='BatySalarie')
        if len(liste_salarie):
            return self.getObjFromUid(liste_salarie[0]).getHeure()
        else:
            return getToolByName(self,'baty_tool').get_property('default','heure_salarie_default')

    def compute_prime_by_salarie(self):
        """retourne les primes de chaque salarie"""
        dict_salarie={}
        for (id,obj) in self.objectItems():
            prime=obj.getObjectFromAttr('tache')
            if not dict_salarie.has_key(obj.getSalarie()):
                dict_salarie[obj.getSalarie()]=0
            if prime.isPrime():
                dict_salarie[obj.getSalarie()]=dict_salarie.get(obj.getSalarie(),0)+obj.getQuantite()*prime.getPrix()
        return dict_salarie
        
                
                

registerType(BatyJournalHeure)


class BatyJournalLigne(BatyObject,):
    portal_type = meta_type = 'BatyJournalLigne' 
    archetype_name = 'BatyJournalLigne'   #this name appears in the 'add' box

    security=ClassSecurityInfo()
    
    schema=BaseSchema +Schema((

        StringField('id',
                required=0,
                mode="",
                accessor="getId",
                mutator="setId",
                default=None,
                widget=IdWidget(label_msgid="label_name",
                                description_msgid="help_name",
                                i18n_domain="plone"),
                ),

        StringField('title',
                required=0,
                searchable=1,
                mode="",
                default='',
                accessor='Title',
                widget=StringWidget(label_msgid="label_title",
                                    description_msgid="help_title",
                                    i18n_domain="plone",
                                    visible=0),
                ),
        

        
        ReferenceField('document',
                       required=0,
                       relationship='document',
                       vocabulary="getListDocument",
                       allowed_types=('None',),
                       mutator='setDocument',
#                       write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                       widget=ReferenceWidget(description_msgid='document_description',
                                              label_msgid='ref_document',
                                              i18n_domain='archbaty',
                                              macro="macro_document"),

                    ),
                
        
        ReferenceField('affaire',
 #                      write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                       required=1,
                       relationship='affaire',
                       validators='validStock',
                       vocabulary="getListAffaire",
                       allowed_types=('None',),
                       edit_accessor='get_last_affaire',
                       
                       widget=ReferenceWidget(description='Code affaire', label='Code Affaire',macro="macro_affaire"),

                    ),
         
         

        ReferenceField('article',
  #                     write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                       required=  1 ,
                       searchable=1,
                       relationship='article',
                       vocabulary='getListArticle',
                       allowed_types=('None',),
                       edit_accessor='get_last_article',
                       
                       
                       
                       widget=ReferenceWidget(description='Veuillez selectionner l\'article' ,
                                              label='Article',
                                              description_msgid="help_saisie_article",
                                              i18n_domain="archbaty",
                                              label_msgid="label_article",
                                              macro="macro_article"),

                       ),
        
        BooleanField('stock',

                    default=0,
                    
                    validators='validStock',
                    mutator='setStock',
                     edit_accessor='get_last_stock', 
                    widget= BooleanWidget(label_msgid="label_sortie_stock",
                                          description_msgid="help_sortie_stock",
                                          i18n_domain="archbaty",
                                          macro="macro_stock"),

                    ),

        FloatField('quantite',
   #                write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                    required=1,
                    
                    default=0.0,
                    validators="validQuantite",
                    edit_accessor='get_last_quantite',
                 
                    widget= DecimalWidget(label="Quantite",
                                          description="Quantite",
                                          macro="macro_left",
                                          label_msgid="label_quantite",
                                          description_msgid="help_quantite",
                                          i18n_domain="archbaty",
                                          size="5"),

                    ),



        

        FloatField('prix_unitaire',
    #               write_permission=BatyPermissions.EDIT_BATY_JOURNAL,
                    required=1,
                    
                    
                    edit_accessor="get_last_prix_unitaire",
                    
                    widget= DecimalWidget(label="Prix unitaire",
                                          description="Prix unitaire",
                                          label_msgid="label_prix_unitaire",
                                          description_msgid="help_prix_unitaire",
                                          i18n_domain="archbaty",
                                          
                                          macro="macro_euro"),

                    ),

#         FloatField('coefficient',
                   
#                     required=1,
#                     searchable=1,
                    
#                     edit_accessor="get_last_coefficient",
#                     widget= DecimalWidget(label="Coefficient de structure",description="Coefficient de structure", macro="macro_percent"),

#                     ),

        ComputedField('prix_total',
                      
                   mode='r',
                   expression="""getattr(context,'prix_unitaire',0)*getattr(context,'quantite',0)""",
                   widget=StringWidget(label="Prix total",
                                       label_msgid="prix_total",
                                       description_msgid="help_prix_total",
                                       i18n_domain="archbaty",
                                       macro="macro_euro"
                                       
                                       ),   
                      ),
        ComputedField('prix_total_journal_article',
                      
                      expression="""context.compute_prix_total_per_article(context.article)""",
                      widget=StringWidget(label="Prix total article",
                                          label_msgid="prix_total_article",
                                          description_msgid="help_prix_total_article",
                                          i18n_domain="archbaty",
                                          macro="macro_euro"),
                     
                      ),
        ComputedField('prix_total_document',
                      expression="""context.compute_prix_total_per_document(context.document)""",
                      widget=StringWidget(label="Prix total document",
                                          label_msgid="prix_total_document",
                                          description_msgid="help_prix_total_document",
                                          i18n_domain="archbaty",
                                          macro="macro_euro",
                                          ),
                      ),
       
        ))




    actions=(
        {
        'id'           : 'view',
        'name'         : 'Ligne journal',
        'action'       : 'string:base_view',
        'permissions'  : (BatyPermissions.VIEW_BATY_JOURNAL, ),
        'category'      : 'object',

	},
        {
        'id'           : 'edit',
        'name'         : 'Edit',
        'action'       : 'string:base_edit',
        'permissions'  : (BatyPermissions.EDIT_BATY_JOURNAL, ),
        'category'      : 'object',
        
	},
        )


    #Methods
    
    
    def get_last_ligne(self):
        portal_catalog=getToolByName(self,'portal_catalog')
        query={'Type':'BatyJournalLigne','path':'/'.join(self.getParentNode().getPhysicalPath())}
        br=portal_catalog.search(query,sort_index='modified',reverse=1,limit=1)
        if len(br):
            return br[0].getObject()
        else:
            return None

    def get_last_attribute_value(self,attribute):
        """ retourne la valeur de l'attribut si il est definit ou la derniere valeur inscrtite au journal """
        obj=self.get_last_ligne()
        if hasattr(self,attribute):
            val=getattr(self,attribute,0)
            #        log('retourne qauantite',val)
            if val:
            ## on est en modif
            
                return val
            else:
                
                return hasattr(obj,attribute) and getattr(obj,attribute) 
            
        
        return

    

    def get_last_quantite(self):
        """ return la derniere quantite """
#        log('retourne qauantite',getattr(self,'quantite',0))
        
        return self.get_last_attribute_value('quantite')

    def get_last_prix_unitaire(self):
        """ retourne le dernier prix unitaire """
        log(self.getParentNode().getFirstArticle())
        return self.get_last_attribute_value('prix_unitaire') or self.getParentNode().getFirstArticle() and self.getParentNode().getFirstArticle().getPrix()
    
 
    def get_last_coefficient(self):
        """ retourne le dernier coef """
        return self.get_last_attribute_value('coefficient') or self.getParentNode().getFirstArticle() and self.getParentNode().getFirstArticle().coefficient_value()
        
        
    def get_last_article(self):
        """ retourne le dernier article """
        return self.get_last_attribute_value('article') or self.getParentNode().getFirstArticle() and self.getParentNode().getFirstArticle().UID()
        
    def get_last_affaire(self):
        """ retourne la derniere affaire """
        return self.get_last_attribute_value('affaire')

    def get_last_stock(self):
        return self.get_last_attribute_value('stock') or self.getParentNode().getFirstArticle() and self.getParentNode().getFirstArticle().getStock()
    

    def setDocument(self,document_uid):
        """ enregistre le document """
        #log(document_uid)
        if document_uid==None:
            setattr(self,'document',None)
        else:
            setattr(self,'document',document_uid)
     

    
                 
    def compute_prix_total_journal_article(self):
        """ return le prix total de l'article en cours """
        return """pass encore implementer"""

    def compute_prix_total_document(self):
        """ retourne le total en reference au document """
        return """pas encore implementer"""
    

    def getPrix_unitaire(self):
        """ retourne le prix unitaire par defaut """

        if(getattr(self,'article',0) and not(getattr(self,'prix_unitaire',0))):
            ## on recupere le prix de l'article
            archetype_tool = getToolByName(self, TOOL_NAME)
            ## on recupere l'uid
            article=archetype_tool.lookupObject(uid=getattr(self,'article'))
            ## on renvoi son prix
            log("on passe la")
            setattr(self,'prix_unitaire',article.getPrix())
            log(article.getPrix())
            return article.getPrix()
        else:
            return getattr(self,'prix_unitaire')

    def getCoefficient(self):
        """ retourne le coefficient """

        if(getattr(self,'article') and not(getattr(self,'coefficient',0))):
            ## on recupere le coeffient de l'article
            archetype_tool = getToolByName(self, TOOL_NAME)
            ## on recupere l'uid
            article=archetype_tool.lookupObject(uid=getattr(self,'article'))
            
            ## on recupere le coef en question
            
            
            setattr(self,'coefficient',article.getCoefficient_value())
            log("j'appelle getCoef",article.getCoefficient_value())
            return article.getCoefficient_value()
        else:
            return getattr(self,'coefficient')
    
        
        



    def compute_total_article(self):
        """ retourne le total du prix l'article """
        return 0
        return getattr(self,'quantite')*getattr(self,'prix_unitaire')

    


    def getMouvement(self):
        """ retourne le movement de stock """
        return hasattr(self,'mouvement') and getattr(self,'mouvement') or 0.0

    def setMouvement(self,value):
        """ mis a jour du mouvement de stock """
        setattr(self,'mouvement',float(value))


    security.declareProtected('setStock',BatyPermissions.EDIT_BATY_JOURNAL)
    def setStock(self,value):
        """ enregistre une quantite :
        
        Lors d'une saisie d'une ligne dans le journal ana:
	     Si l'article sort du stock :
                1 verication du stock disponible (par rapport a la quantite virtuel de stock qu'est le stock theorique)
                2 si ok decrementation du stock theorique

	Lors d'une modification d'une ligne non mouvement:

             Si l'article sort du stock  et si il etait deja en sortie de stock alors:
	        1 verif du stock dispo par rapport a la difference de la quantite d'avant et la quantite d'apres
                 ex : si avant la modif j'avais 10 et apres j'ai 15 alors je doit diminu mon stock de 5 donc je regarde si mon
                 stock theorique peut supporter cette diminution
                2 diminution de la difference de quantite au niveau du stock theorique

	     Si l'article sort du stock  et si il n'etait pas en sortie de stock alors:
               meme cas qu'en creaton

	     Si l'article ne sort pas du stock  et si il etait en sortie de stock alors:
                augmentation du stock theorique puisque on ne le sort pas du stock


	     Si l'affaire est le stock alors il faut faire l'inverse et uniquement en entre car pas de sortie de stock possible.
        
        """



        
        
        if not value:
            return
        if self.REQUEST is not None:
            
            """ il y a forcement sortie ou entree de stock """
            REQUEST=self.REQUEST
            
            uid_article=REQUEST.get('article')
            uid_affaire=REQUEST.get('affaire')
            qt=REQUEST.get('quantite')
            
            obj=self.archetype_tool.getObject(uid_article)
            aff=self.archetype_tool.getObject(uid_affaire)

            if obj is None and aff is None:
                ### uniquement pour les tests
                setattr(self,'stock',value)
                return
            
            #mvt=getattr(self,'mouvement',0) and float(getattr(self,'mouvement',0)) or 0.0
            mvt=self.getMouvement() or 0.0
            ## on annule le dernier mouvement
            log('maj stock theo',mvt)
            obj.maj_stock_theorique(-mvt)

            stock=value and int(value) or 0
            quantite=0.0
            if stock:
            ## on calcule le nouveau mouvement
                if isAffaireStock(aff):
                    quantite=- float(qt)
                else:
                    quantite=+ float(qt)

            log("mouvement de stock",quantite)
            obj.maj_stock_theorique(quantite)
            log('mis a jour du stock',quantite)
            self.setMouvement(quantite)
            #setattr(self,'mouvement',quantite)
            
            
                
        setattr(self,'stock',value)
    
        
    
    def manage_beforeDelete(self,*args,**kw):
        """ hook for teorical stock """
        log("je delete %s %s" % (str(args),str(kw)))
        ## on enleve la  quantite theorique si on doit le faire
        

        
        
        obj=self.archetype_tool.getObject(getattr(self,'article',None))
        obj.maj_stock_theorique(-self.getMouvement()) 
                
        for base in BatyJournalLigne.__bases__:
            log("ja appelle manage_beforeDelete %s" % str(base) )
            callbase(self, base,'manage_beforeDelete',args,kw,1)

#    def manage_afterAdd(self,*args,**kw):
#        """ hook for teorical stock """
#        log("ajout de %s",self.id)
#         if int(getattr(self,'stock',0))==1:
#             log("je passe ici ddddd")
#             obj=self.archetype_tool.getObject(getattr(self,'article',None))
#             if float(obj.getStock_theorique) - float((getattr(self,'quantite',0))) >0:
#                 obj.maj_stock_theorique(getattr(self,'quantite',0))
#             else:
#                 setattr(self,'quantite',0)
                
                
            
            

        
registerType(BatyJournalLigne)# end class BatyArticle


class BatyJournalHeureLigne(BatyObject):
    portal_type = meta_type = 'BatyJournalHeureLigne' 
    archetype_name = 'BatyJournalHeureLigne'   #this name appears in the 'add' box

    security=ClassSecurityInfo()
    
    schema=BaseSchema +  Schema(( BatyFields.date_journal(),BatyFields.id(),BatyFields.hidden_title(),BatyFields.reference_salarie(),BatyFields.reference_affaire(),BatyFields.reference_tache(),BatyFields.quantite_tache(),BatyFields.prix_tache()))


registerType(BatyJournalHeureLigne)
