#!/usr/bin/python
import os
from time import time

import sys
try:
	from reportlab.platypus import *
	from reportlab.lib.styles import getSampleStyleSheet
	from reportlab.rl_config import defaultPageSize
	from reportlab.lib.units import inch
	from reportlab.lib.pagesizes import landscape, portrait
	from reportlab.lib import colors
except:
	Paragraph='Para'
        tstyle='tstyle'
import string, cStringIO
try:
	from PIL import Image as PIL_Image
except:
	pass
from Products.ArchBaty.properties import BatyFields
from BatyObject import BatyObject
from AccessControl import ClassSecurityInfo
from Products.ArchBaty import BatyPermissions
from Products.Archetypes.public import registerType, Schema
from Products.ArchBaty.properties import BatyFields
from OFS.Image import File


### style sheet
try:
	styles = getSampleStyleSheet()

	tstyle = TableStyle([
            
            ('GRID', (0,0), (-1,-1), 0.25, colors.silver),
            ('BOX',(0,0),(-1,0),1,colors.silver),
            ('BACKGROUND', (0,0),(-1,0),colors.snow),
            
            ('ALIGN', (0,0), (-1,0), 'CENTER'),
            ('ALIGN', (0,0), (-1,-1), 'RIGHT'),
            ])
except:
	pass

def getImageFromZODB(context, name) :
    """Retrieves an Image from the ZODB, converts it to PIL,
    and makes it 0.75 inch high.
    """
    try :
        # try to get it from ZODB
        logo = getattr(context, name)
    except AttributeError :
        # not found !
        return None
    
    # Convert it to PIL
    image = PIL_Image.open(cStringIO.StringIO(str(logo.data)))
    (width, height) = image.size
    
    # scale it to be 0.75 inch high
    multi = ((height + 0.0) / (0.75 * inch))
    width = int(width / multi)
    height = int(height / multi)
    return ((width, height), image)


DOC_BOOK_COMMAND="""jw -f docbook -o /tmp/ -b %s %s"""

class DocBookReport :
    """ genere un docbook """
    

    
    def __init__(self,content):
        self.content=content
        

    def generate_report(self,type_report='pdf',*arg):
        """ genere un rapport type docbook """
        xml_name="xml_%s" %(str(time()))
        xml=open(xml_name,'a')
        xml.write(self.content)
        xml.close()
        os.system(DOC_BOOK_COMMAND % (type_report,xml_name))
        
class BatyReport (BatyObject,):
    """ class responsable de la generation pdf des rapports """

    portal_type = meta_type = 'BatyReport'
    archetype_name='BatyReport'
    schema=BatyObject.schema + Schema((BatyFields.pdf_file(),))

    security=ClassSecurityInfo()
    
    actions=(
        {
        'id'           : 'view_pdf',
        'name'         : 'rapport',
        'action'       : 'string:${object_url}/generate_report',
        'permissions'  : (BatyPermissions.VIEW_BATY_CONTENT, ),
        'visible' : 1,
        },
	)
    
    def __init__(self,id,title='',logo=None,header_left='',header_right='',draw_line=0,landscape=0):
        """ construteur d'un rapport """
        BatyObject.__init__(self,id)
        self.Elements=[]
        self.styles=getSampleStyleSheet()
        self.HeaderStyle=self.styles["Heading1"]
        self.ParaStyle=self.styles["Normal"]
        self.PreStyle=self.styles["Code"]
        self.landscape=landscape
        self.logo=logo
        self.header_left=header_left
        self.header_right=header_right
        self.draw_line=draw_line
        self._v_report=None
        
        
     
    security.declarePrivate('batyreport_Pages')
    def batyreport_Pages(self,canvas,doc):
        self.batyreport_haut_de_page(canvas,doc)
        self.batyreport_pied_de_page(canvas,doc)

    security.declarePrivate('batyreport_pied_de_page')
    def batyreport_pied_de_page(self,canvas,doc):
        canvas.saveState()
        canvas.setFont('Times-Roman',9)
        canvas.drawString(inch, 0.75 * inch, "Page %d" % (doc.page))
        canvas.restoreState()


    security.declarePrivate('batyreport_haut_de_page')
    def batyreport_haut_de_page(self,canvas,doc):
        canvas.saveState()
        canvas.setFont('Times-Roman',9)

        if self.logo:
            ## le logo est une PIL image
            ## on la reduit  0.75 inch
            (width, height) = self.logo.size
            multi = ((height + 0.0) / (0.75 * inch))
            width = int(width / multi)
            height = int(height / multi)
            canvas.drawImage(self.logo,inch,doc.pagesize[1] - 0.90*inch,width,height)
        elif self.header_left:
            canvas.drawString(inch,doc.pagesize[1] - 0.90*inch ,self.header_left)    
        if self.header_right:
            canvas.drawRightString(doc.pagesize[0]-inch,doc.pagesize[1] - 0.90*inch ,"Ceci est le haut de la page a gauche")
        if self.draw_line:
            canvas.line(inch,doc.pagesize[1] - inch,doc.pagesize[0] - inch,doc.pagesize[1] - inch)
        
        canvas.restoreState()

    security.declareProtected(BatyPermissions.EDIT_BATY_CONTENT,'batyreport_p')
    def batyreport_p(self,txt):
        return self.batyreport_header(txt, style=self.ParaStyle, sep=0.1)

    
    security.declareProtected(BatyPermissions.EDIT_BATY_CONTENT,'batyreport_pre')
    def batyreport_pre(self,txt):
        s = Spacer(0.1*inch, 0.1*inch)
        self.Elements.append(s)
        p = Preformatted(txt, self.PreStyle)
        self.Elements.append(p)



        security.declareProtected(BatyPermissions.EDIT_BATY_CONTENT,'batyreport_header')
    def batyreport_header(self,txt, style=None,klass=Paragraph, sep=0.3):
        if style is None:
            style=self.HeaderStyle
        
        s = Spacer(0.2*inch, sep*inch)
        
        self.Elements.append(s)
        para = klass(txt, style)
        self.Elements.append(para)


    security.declareProtected(BatyPermissions.EDIT_BATY_CONTENT,'batyreport_page_break')
    def batyreport_page_break(self):
        self.Elements.append(PageBreak())
    

    
    security.declareProtected(BatyPermissions.EDIT_BATY_CONTENT,'batyreport_table')
    def batyreport_table(self,data=[],hAlign = "LEFT",tstyle=tstyle):
        s = Spacer(0.1*inch, 0.1*inch)
        self.Elements.append(s)
        #if col is None:
        #    l =len(data[0])
        #col=[60]*l
        t= Table(data, None,None)
        

        sty = tstyle
        
        t.setStyle(sty)
        
        t.hAlign=hAlign
        self.Elements.append(t)
    



    security.declareProtected(BatyPermissions.VIEW_BATY_CONTENT,'generate_report')
    def generate_report(self,context=None):
        """ genere un rapport """

        self._v_report=None
        if self.landscape:
            pagesize=(defaultPageSize[1],defaultPageSize[0])
        else:
            pagesize=(defaultPageSize[0],defaultPageSize[1])
        self._v_report= cStringIO.StringIO()
        ### construction en memoire 
        doc = SimpleDocTemplate(self._v_report,pagesize=pagesize)
        Elements=list(self.Elements)
        
        doc.build(Elements, onFirstPage=self.batyreport_Pages,onLaterPages=self.batyreport_Pages)
        
        

        self.getField('pdf_file').set(self, self._v_report.getvalue() , mimetype='application/pdf')
        self._p_changed=1
        return self.getPdf_file()

    def getPdf_file(self):
        """ retourne le contenu du fichier """
        if self.REQUEST is not None:
            if hasattr( self.REQUEST,'RESPONSE'):
                response=self.REQUEST.RESPONSE
                self.REQUEST.RESPONSE.setHeader('Content-Type', 'application/pdf')
                self.REQUEST.RESPONSE.setHeader('Content-Disposition', 'attachment; filename=%s' % self.getId())
            
        try:
            return str(getattr(self,'pdf_file'))
        except AttributeError:
            return self.generate_report()
                
        

registerType(BatyReport)
    
    

if __name__ == '__main__':
#    fd=open('/home/youenn/docbook/book.sgm')
#    content=fd.read()
    
#    r=Report(content)
    
#    print r.content
#    r.generate_report()

    from  PIL import Image
    logo=Image.open('logo.jpg')
    pr=PythonReport(landscape=1,logo=logo,header_right='baty rapport',file_name='with_logo.pdf')
    pr.header('Baty Report')
    pr.p("""ceci est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapide""")
    pr.header('Section')
    pr.p("""ceci est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapide ceci est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive""")

    data=[['unddddd dd','deux','trois','quatre','cinq'], range(5),range(5),range(5),range(5)]
    print data

    pr.table(data)

    pr.page_break()
    pr.header('Page 2')
    pr.p("""    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapidececi est un paragraph baty <i>et vive l'italique</i> et vive
    <b>vive le gras </b> et ca doit etre bien plus rapide""")
    
    
    pr.go()
         
    
