from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName 

from Products.Archetypes.debug import log, log_exc
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Acquisition import aq_parent, aq_inner, aq_base
from Products.CMFPlone.PropertiesTool import PropertiesTool
from BatyException import BatyException
from properties.BatyPropertySheet import *
from Products.ArchBaty import BatyPermissions
from baty_types.BatyAffaire import isAffaireStock,BatyFraisGeneraux
from baty_types.BatyArticle import BatyMouvement
from baty_types.BatyJournal import BatyJournalLigne
from baty_types.BatyBase import safe_cmp
from baty_types.BatyReport import BatyReport
from BatyUtils import generate_id, nextPeriode
from DateTime import DateTime

from Products.CMFCore.ActionInformation import oai
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import createExprContext
from Products.Archetypes.config import TOOL_NAME

def addProperty(context,klass={}):
    for key in klass.keys():
        context._setObject(key,klass[key]())


def addtoattr(obj,attr,value):
    log('addtoattr avant',obj.title_or_id(),attr,value,getattr(obj,attr,0))
       
    setattr(obj,attr,getattr(obj,attr,0)+value)

    log('addtoattr apres',obj.title_or_id(),attr,value,getattr(obj,attr,0))

def get_mvt_object(affaire,article):
    """ creer un fichier mouvement si il n'exide pas """
    mvt_id='mvt_'+article.getId()
    mvt_title='mvt_'+article.Title()
    if mvt_id not in affaire.objectIds():
        ## il n'existe pas il faut le cree ##
        mvt_object=BatyMouvement(mvt_id,title=mvt_title)
        setattr(mvt_object,'article',article.UID())
        setattr(mvt_object,'affaire',affaire.UID())
        
        affaire._setObject(mvt_id,mvt_object)
        log('ajout baty mouvement', mvt_id)
    
    return affaire[mvt_id]
    

def get_periode_id(month):
    if int(month)/2==0:
        return 'pair'
    else:
        return 'impair'



class BatyTool(PropertiesTool):
    """ Store global variable for baty types via PropertySheets instance (basetool)"""
    id       ='baty_tool'
    meta_type='Baty Tool'
    
    security=ClassSecurityInfo()

    
    manage_options=(
        PropertiesTool.manage_options
        )


    def __init__(self):
        addProperty(self,
                    klass={'tva':BatyPropertyTVA,
                           'coef': BatyPropertyCoefStruct,
                           'coef_stockage' : BatyPropertyCoefStockage,
                           'compte_exploitation':BatyPropertyCompteExploitation,
                           'unite': BatyPropertyUnite,
                           'secteur': BatyPropertySecteur,
                           'famille': BatyPropertyFamille,
                           'compta': BatyPropertyPlanComptable,
                           'default' :BatyDefaultProperty,
                           }
                    )
        

    security.declarePublic('get_property')
    def get_property(self,section,key,default=None):
        """ retourne une propriete par defaut """
        if self[section].hasProperty(key):
            return self[section].getProperty(key)
        else:
            return default





    security.declarePublic('listElementsFromUid')
    def listElementFromUid(self,uid,catalog=None,filter={}):
        """ get all content element in the element refrenced by uid by searching in the catalog (catalog arg)"""
        catalog=catalog is None and getToolByName(self,'portal_catalog') or catalog
        ps=catalog.search({'UID':uid})
        if len(ps) == 1:
            ## path est le chemin de recherhe de l'element desire
            path_ps='/'.join(ps[0].getObject().getPhysicalPath())
            
            # maintenant on renvoie les element correspondant  l'ensememble des elements contenu dans l'objet reference par l'uid
            filter.update({'path':path_ps})
            return catalog.search(filter)
        else:
            return []
        

    security.declarePublic('getCatalogTool')
    def getCatalogTool(self,catalog_name='portal_catalog'):
        """ return the catalog tool """
        return getToolByName(self,catalog_name)
            
    security.declarePublic('getMetadataFromUid')
    def getMetadataFromUid(self,uid,meta,catalog_name='portal_catalog'):
        """ in : an uid , a key to a metadata
            out : give the value of metadata if search in catalog suceed """
        br=self.getCatalogTool(catalog_name).search({'UID':uid})
        if len(br)==1:
            return getattr(br[0],meta)
        else:
            return None

    security.declarePublic('getArchetypeTool')
    def getArchetypeTool(self):
        """ get archetype tool """
        return getToolByName(self, TOOL_NAME)

    security.declarePublic('getObjFromUid')
    def getObjFromUid(self,UID=None ):
        """ return obj from his uid """ 
        if UID is not None:
            return self.getArchetypeTool().getObject(UID) 
        else:
            return None

    security.declarePublic('getMetadataFromAttr')
    def getMetadataFromAttr(self,instance,attr,meta,catalog_name='portal_catalog'):
        """ return the value of attr of ref instance of an object """
        uid=getattr(instance,attr)
        
        return self.getMetadataFromUid(uid,meta,catalog_name=catalog_name) 



    

    security.declarePublic('listFilteredActionsFor')
    def listFilteredActionsFor(self,object=None):
        """ retourne les actions de l'outil baty_tool
        pour optimiser la construction du menu
        """
        portal = aq_parent(aq_inner(self))
        
        if object is None or not hasattr(object, 'aq_base'):
            folder = portal
        else:
            folder = object
            # Search up the containment hierarchy until we find an
            # object that claims it's a folder.
            while folder is not None:
                if getattr(aq_base(folder), 'isPrincipiaFolderish', 0):
                    # found it.
                    break
                else:
                    folder = aq_parent(aq_inner(folder))
        
        actions = []
        append=actions.append

        portal_action=getToolByName(self,'portal_actions')

        ec = createExprContext(folder, portal, object)
        info= oai(portal_action,folder,object)

        portal_action._listActions(append,self,info,ec)


        filtered_actions={}
        for action in actions:
            category = action['category']
            permissions = action.get('permissions', None)
            visible = action.get('visible', 1)
            if not visible:
                continue
            verified = 0
            if not permissions:
                # This action requires no extra permissions.
                verified = 1
            else:
                # startswith() is used so that we can have several
                # different categories that are checked in the object or
                # folder context.
                if (object is not None and
                    (category.startswith('object') or
                     category.startswith('workflow'))):
                    context = object
                elif (folder is not None and
                      category.startswith('folder')):
                    context = folder
                else:
                    context = portal
                for permission in permissions:
                    # The user must be able to match at least one of
                    # the listed permissions.
                    if _checkPermission(permission, context):
                        verified = 1
                        break
            if verified:
                catlist = filtered_actions.get(category, None)
                if catlist is None:
                    filtered_actions[category] = catlist = []
                # Filter out duplicate actions by identity...
                if not action in catlist:
                    catlist.append(action)
                # ...should you need it, here's some code that filters
                # by equality (use instead of the two lines above)
                #if not [a for a in catlist if a==action]:
                #    catlist.append(action)

        return filtered_actions

    ## alias
    security.declarePublic('listFilteredActions')
    listFilteredActions=listFilteredActionsFor
        


    security.declarePublic('reindex_baty_article')
    def reindex_baty_article(self):
        """reindex tout les article de baty"""
        catalog = getToolByName(self, 'portal_catalog')
        results = catalog({'Type':'BatyArticle'})
        for item in results:
            try:
                item.getObject().reindexObject()
                #print "reindex "+ item.getId +" :OK"
            except:
                #print "reindex "+ item.getId +" :KO"
                log('error' + str(item.getRID()) +' n\'est plus correct')
            
        
    

    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'genere_rapport_journal' )
    def genere_rapport_journal(self,context):
        """ genere le rapport du journal """
        if isinstance(context,BatyJournalLigne):
            return
            
        archetype_tool=getToolByName(self,'archetype_tool')
        baty_rapport=BatyReport('rapport_journal',title= context.Title(),header_left= context.Title(),draw_line=1,landscape=1)

        baty_rapport.batyreport_header('Rapport : ' + context.Title(),)
        baty_rapport.batyreport_p("""<i>id:</i> <b>%s</b>""" % (context.getId(),))
        baty_rapport.batyreport_p("""<i>Periode:</i> <b>%s</b>""" % (context.getPeriode(),))
        baty_rapport.batyreport_p("""<i>Date de saisie:</i><b>%s</b>""" % (getattr(context,'date_de_saisie').strftime("%d/%m/%Y"),))
        baty_rapport.batyreport_p("""<i>Entreprise:</i> <b>%s</b>""" % (context.getEnt().Title(),))

        ####Priode Date cration Date de saisie
        #N Ligne N Document N Affaire Nom NArticle Nom 
        #Quantit Prix U Code frais indirect Mvt de Stock Mise a jour

        data_header=['N ligne', 'N Document',  'Affaire', 'Article','Quantit', 'Prix u', 'Frais', 'Mvt de Stock']

        list=context.objectItems('BatyJournalLigne')
        list=[x[1] for x in list]
        list.sort(lambda x, y: safe_cmp(getattr(x,'creation_date'),getattr(y,'creation_date')))
        
        i=0
        courante_date_saisie=''
        tup=[]
        for item in list:
            date_saisie=getattr(item,'creation_date')
            if not courante_date_saisie or courante_date_saisie.strftime('%Y%m%d') != date_saisie.strftime('%Y%m%d'):
                log("dans le if ",courante_date_saisie)
                courante_date_saisie=date_saisie
                if len(tup):
                    
                    baty_rapport.batyreport_table(tup)
                    baty_rapport.batyreport_page_break()
                    tup=[data_header]
                    baty_rapport.batyreport_header('Saisie du %02d/%02d/%04d' % (courante_date_saisie.day(),courante_date_saisie.month(),courante_date_saisie.year()))
                else:
                    ## c'est juste le debut
                    tup=[data_header]
                    baty_rapport.batyreport_header('Saisie du %02d/%02d/%04d' % (courante_date_saisie.day(),courante_date_saisie.month(),courante_date_saisie.year()))
            document=item.getDocument()

            log("on ajoute %d %s" % (i,archetype_tool.getObject(document)))
            article=archetype_tool.getObject(item.getArticle())
            coefficient=article.coefficient_value()
            l_item=[i,item.getDocument() and archetype_tool.getObject(item.getDocument()) or '-',archetype_tool.getObject(item.getAffaire()).Title(),archetype_tool.getObject(item.getArticle()).Title(),item.getQuantite(),item.getPrix_unitaire(),coefficient,'oui' and item.getStock() and int(item.getStock()) or 'non']
            tup.append(l_item)
            i=i+1
        if len(tup):
            ## il faut vider la table
            baty_rapport.batyreport_table(tup)
        context._setObject('rapport_journal',baty_rapport)
        


    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'cloture_journal' )

    def cloture_journal(self,context):
        """ cloture le journal : appell par wrkflow_tool.doActionFor(context,'mouvementer') """
        if isinstance(context,BatyJournalLigne):
            ## c'est une ligne journal
            self.cloture_journal_ligne(context)
        else:
            ## c'est le journal en question
            list=context.objectValues('BatyJournalLigne')
            for item in list:
                ### l'item est une batyjounal ligne
                workflow_tool=getToolByName(self,'portal_workflow')
                if workflow_tool.getInfoFor(item,'state_mouvement') == 'creation':
                    ## rappel cloture_journal avec l'item en question
                    ## algo : tant que j'ai des elements  cloturer je les cloture
                    workflow_tool.doActionFor(item,'mouvementer')
            self.genere_rapport_journal(context)
            
        

    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'cloture_journal_ligne' )
    def cloture_journal_ligne(self,context):
        """ cloture une ligne journal """

        ## definition des variables
        container=context.getParentNode()
        entreprise=container.getEnt()
        archetype_tool=getToolByName(self,'archetype_tool')
        periode_id=container.get_periode_type()
        #month=periode[0:2]

        ## recuperation des articles frais generaux ,credit frais generaux, frais de stockage
        basearticle=entreprise.getBaseArticle()
        ## on a la base frais generaux
        fg_affaire= entreprise.getFirstObjectForType(type='BatyFraisGeneraux')
        st_affaire= entreprise.getFirstObjectForType(type='BatyStock')

        article=archetype_tool.getObject(context.getArticle())
        
        affaire=archetype_tool.getObject(context.getAffaire())
         
        frais_generaux = article.getObjectFromAttr('famille').getObjectFromAttr('coefficient').getObjectFromAttr('article_frais')
        credit = article.getObjectFromAttr('famille').getObjectFromAttr('coefficient').getObjectFromAttr('article_frais')

        
        frais_stockage = article.getObjectFromAttr('famille').getObjectFromAttr('coefficient_stockage').getObjectFromAttr('article_frais')

        ## recuperation des affaires stocks et frais general

            
        
            
       ## c'est le conteneur qui va contenir l'objet


        coefficient = article.getCoefficientValue()
        log("init",basearticle,fg_affaire,frais_generaux,credit,frais_stockage ,periode_id,coefficient)

        if affaire is None:
            raise BatyException(BatyException.BATY_JOURNAL_ERREUR_AFFAIRE)
        if article is None:
            raise BatyException(BatyException.BATY_JOURNAL_ERREUR_ARTICLE)
        if frais_generaux is None:
            raise BatyException(BatyException.BATY_JOURNAL_ERREUR_ARTICLE_FG)
        if fg_affaire is None:
            raise BatyException(BatyException.BATY_JOURNAL_ERREUR_AFFAIRE_FG)


            
        ## on ajoute les quantite mois
        #prix_total=float(item.getPrix_unitaire()) * float(item.getQuantite())
        #####---------------- Debit article ----------------------#####
        
        if context.getMouvement() < 0:
                ## entre de stock
            type_mvt='entre'
        else:
            type_mvt='sorti'

        log("------------------------ mouvement article ----------------------") 

        self.mvt_article(article,affaire,float(context.getPrix_unitaire()),float(context.getQuantite()),float(coefficient),type_mvt,periode_id,frais_stockage,frais_generaux,credit,st_affaire,fg_affaire)

        log("------------------------ fin  mouvement article ----------------------") 

        ### si entre mis a jour du prix moyen pondere
        if type_mvt == 'entre':

            log('------------------calcul du prix moyen ponderer ------------')
        
            ancien_pmp=article.getPrix_moyen_ponderer()
            ancienne_quantite=article.getQuantite()

            nouveau_pmp=(float(ancien_pmp)*float(ancienne_quantite) + float(context.getQuantite())*float(context.getPrix_unitaire()))/(float(ancienne_quantite)+context.getQuantite())
            setattr(article,'prix_moyen_ponderer',nouveau_pmp)
                
            ## mis a jour du stock du prix d'achat et du prix de vente
            addtoattr(article,'quantite',context.getQuantite())
            setattr(article,'dernier_prix_achat',context.getPrix_unitaire())
            setattr(article,'prix_vente', float(context.getPrix_unitaire()) * ( 1 + float(coefficient)/100.0) )
            log('calcul du prix moyen ponderer',article.Title(),nouveau_pmp,article.getPrix_vente())

            log('------------------fin calcul du prix moyen ponderer ------------')
                
        else:
            ### sortie
            log('------------- sortie de stock ----------------------------')
                
            sortie_stock=context.getStock() and int(context.getStock()) or 0
            if sortie_stock:
                #mvt_object_stock=get_mvt_object(st_affaire,article)
                ## on mouvemente avec les quantites en negatif avec l'article et l'affaire stock
                          
                #"""ref spec : Si L'article sort du stock, le programme cre un enregistrement supplmentaire avec comme affaire, l'affaire stock, les quantits en ngatif, et le N ligne +1. La quantit en stock sera diminu de cette quantit sortie ..Il n'y a pas de code frais gnraux La quantit de l'article sera ajoute aux quantits sorties mois , exercice"""
                
                mvt_object=get_mvt_object(st_affaire,article)
                
                log("debit article",  affaire.getId(),  article.getId())
                quantite=-float(context.getQuantite())
                addtoattr(mvt_object,'prix_mois_%s' % periode_id,float(context.getPrix_unitaire())*quantite)
                addtoattr(mvt_object,'quantite_mois_%s' % periode_id,quantite)
                addtoattr(mvt_object,'quantite_exercice',quantite)
                addtoattr(mvt_object,'prix_exercice',float(context.getPrix_unitaire())*quantite)
                ## misa jour du stock 
                addtoattr(article,'quantite',quantite)

                
            log('------------- sortie de stock ----------------------------')

            
            

            

            


    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'mvt_article' )

    def mvt_article(self,article,affaire,prix,quantite,coefficient,type_mvt,periode_id,frais_stockage,frais_generaux,credit,st_affaire,fg_affaire):
    
        prix_total=prix*quantite
        prix_total_frais=prix_total*(coefficient/100.0)
        
        mvt_object=get_mvt_object(affaire,article)

        
        log("****************** PHASE 1 Debit article************************")
        ### debit article ####
        log("debit article",  affaire.getId(),  article.getId(),  prix_total)         
        addtoattr(mvt_object,'prix_mois_%s' % periode_id,prix_total)
        addtoattr(mvt_object,'quantite_mois_%s' % periode_id,quantite)
        addtoattr(mvt_object,'quantite_exercice',quantite)
        addtoattr(mvt_object,'prix_exercice',prix_total)


        addtoattr(article,'prix_%s_mois_%s' % (type_mvt,periode_id),prix_total)
        addtoattr(article,'quantite_%s_mois_%s' % (type_mvt,periode_id),quantite)
        addtoattr(article,'quantite_%s_exercice' % (type_mvt),quantite)
        addtoattr(article,'prix_%s_exercice' % (type_mvt) ,prix_total)



        #####---------------- Debit frais generaux ---------------#####
            
        if isAffaireStock(affaire):
            ### l'affaire est de type stock
            frais=frais_stockage
        else:
            
            frais=frais_generaux

        log("****************** PHASE 2 Debit frais generaux ************************")

        
        mvt_frais=get_mvt_object(affaire,frais)
        log("debit frais",  affaire.getId(),  frais.getId(),  prix_total_frais)         

        
        addtoattr(mvt_frais,'prix_mois_%s' % periode_id,prix_total_frais)
        addtoattr(mvt_frais,'prix_exercice',prix_total_frais)

        ## article frais
        addtoattr(frais,'prix_entre_mois_%s' % periode_id,prix_total_frais)
        addtoattr(frais,'prix_entre_exercice' ,prix_total_frais) 

        

        log("****************** PHASE 3 Crebit frais generaux ************************")
        #####---------------- Credit frais generaux --------------#####
            
        mvt_credit=get_mvt_object(fg_affaire,credit)
        log("credit",  fg_affaire.getId(),  mvt_credit.getId(),  prix_total_frais)         
            
        addtoattr(mvt_credit,'prix_mois_%s' % periode_id,prix_total_frais)
        addtoattr(mvt_credit,'prix_exercice',prix_total_frais)

        addtoattr(credit,'prix_entre_mois_%s' % periode_id,prix_total_frais)
        addtoattr(credit,'prix_entre_exercice',prix_total_frais)




    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'mvt_journal_heure_ligne' )
    def mvt_journal_heure_ligne(self,context,create=1):
        """ mouvemente ligne journal des heure """
        """ algo :
        1 on recupere le fichier de mouvement
        2 on met a jour prix_mois_impair, prix_mois_pair, quantite_mois_impair, quantite_mois_pair, quantite_exercice et prix exercice
        3 on met a jour les donnees du salarie :
        heure_pair,prime_pair,salaire_pair,heure_impair,prime_impair,heure_exercice,prime_exercice
        suivante que c'est une tache ou une prime et qu'on est en periode pair ou impair
        4 on met a jour les taches en entree 
        """
        

        ## definition des variables
        journal_heure=context.getParentNode()
        entreprise=journal_heure.getEnt()
        log('mvt_journal_heure_ligne',str(journal_heure))
        periode=journal_heure.getPeriode()
        periode_id=journal_heure.get_periode_type()

     
        tache_or_prime=context.getObjectFromAttr('tache')
        affaire=context.getObjectFromAttr('affaire')
        salarie=context.getObjectFromAttr('salarie')
        
            
        
        if affaire is None or salarie is None or tache_or_prime is None:
            raise BatyException(BatyException.BATY_JOURNAL_ERREUR_AFFAIRE)


            
        
        log("------------------------ mouvement tache ou primme ----------------------") 
        quantite=create and float(context.getQuantite()) or (- float(context.getQuantite()))
        # prix=None
#         if tache_or_prime.isPrime():
#             if quantite > 0:
#                 ## on sauve le prix unitaire de la tache
#                 context.setPrix(tache_or_prime.getPrix()*quantite)
#                 prix=context.getPrix()
#             else:
#                 prix=-context.getPrix()
        
            

        
        
        
        self.mvt_tache_prime(tache_or_prime,affaire,salarie,periode_id,quantite)

        log("------------------------ fin  mouvement article ----------------------") 

        ### maj des objets salaries + taches
        
            
    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'mvt_tache_prime' )
    
    def mvt_tache_prime(self,tache_or_prime,affaire,salarie,periode_id,quantite):
        """ mouvemente une tache ou une prime au niveau d'une affaire
        On ne mouvemente que les quantits au depart
        C'est lors de la cloture du journal que sont mis  jour les prix et les frais generaux

       
        """
        
        
        mvt_object=get_mvt_object(affaire,tache_or_prime)
        type_mvt='entre'
        
        log("****************** PHASE 1 Debit article************************")

        #### maj de l'objet mouvement
        addtoattr(mvt_object,'quantite_mois_%s' % periode_id,quantite)
        addtoattr(mvt_object,'quantite_exercice',quantite)
        


        #### maj de l'objet article
        addtoattr(tache_or_prime,'quantite_%s_mois_%s' % (type_mvt,periode_id),quantite)
        addtoattr(tache_or_prime,'quantite_%s_exercice' % (type_mvt),quantite)

        if not tache_or_prime.isPrime():
            ##### maj  de l'objet salarie pour les heures
            addtoattr(salarie,'heure_%s' % (periode_id,),quantite)
            addtoattr(salarie,'heure_exercice' ,quantite)


            
    


    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'clt_journal_heure' )
    def clt_journal_heure(self,journal,infos_salarie={}):
      
        """ cloture le journal des heures :


        Pour chaque salari il faudra au pralable calculer le prix horaire
	Salaire total / Salaire brut 	donne		le coefficient des charges
	Salaire brut  - primes		donne		le cot des heures payes
	Le cot heur / heures pay	donne		cot horaire brut
	Cot horaire brut x Coeff des charges	cot horaire total

        
        Au moment de la mise a jour de la main d'oeuvre, l'operateur saisit par salarie le salaire Paye S1 (hors charge social) et le salaire global S2 ( salaire paye + charge social salarie et patronal)
        S2/S1=KS

        Prime global = Prime releve * KS
        
        Salaire horaire =( Salaire global - Prime global ) / (total heure)
        
        A ce moment la le systeme valorise l'ensemble des taches par salarie et creer un nouveau journal des heures pour la
        periode suivante

        infos_salaries => dictionnaire des salaries avec leur salaire

        + cloture le journal


        #### affectation des frais generaux
        
         frais generaux :
        si affaire != frais generaux
        alors:
        affectation a l'article frais generaux de la famille de la quantite
        (ie  tache_or_prime.gDebit frais generauxetObjectFromAttr('famille').getObjectFromAttr('article_frais')
        + affectation du Debit frais generaux
        get_mvt_object(affaire,frais) avec affaire = affaire frais generale ( entreprise.getFirstObjectForType(type='BatyFraisGeneraux'))
        
        
        """
        
        log('clt_journal_heure',str(journal))
        periode_id=journal.get_periode_type()

        ### calcul de l'emsemble des primes par salaries
        prime_by_salarie=journal.compute_prime_by_salarie()

        ### pour chaque ligne du journal ###
        
        for (id_ligne,ligne) in journal.objectItems('BatyJournalHeureLigne'):
            ## on recupere le salarie de la ligne
            s_ligne=ligne.getSalarie()
            log('SALARIE',s_ligne,id_ligne,str(infos_salarie))
            ## on regardde au niveau de la saisie de l'utilisateur
            if not infos_salarie.has_key(s_ligne):
                ## il y a un probleme , il manque les infos d'un salarie
                raise BatyException(BatyException.BATY_JOURNAL_ERREUR_SALARIE   )

            info_salarie=infos_salarie[s_ligne]
            obj_salarie=ligne.getObjectFromAttr('salarie')
            ## calcul du coefficient pour calculer la prime global
            ks=float(info_salarie['salaire_global'])/float(info_salarie['salaire_paye'])

                        
            prime_global=prime_by_salarie[obj_salarie.UID()]*ks
            #getattr(obj_salarie,'prime_%s' % (periode_id,),0)*ks
            ## il faut calculer le prix ht pour le salarie
            
            
            if getattr(obj_salarie,'heure_%s' % (periode_id,),0) != 0:
                salaire_horaire_global=(float(info_salarie['salaire_global'])-prime_global)/getattr(obj_salarie,'heure_%s' % (periode_id,),0)
                
                
                ### on valorise la ligne, la ligne mouvement, le salarie et la tache

                tache_or_prime=ligne.getObjectFromAttr('tache')
                
                if tache_or_prime.isPrime():
                    prix_ht=tache_or_prime.getPrix()
                    prix_global=prix_ht*ks*ligne.getQuantite()
                else:
                    prix_global=ligne.getQuantite()*salaire_horaire_global

                ligne.setPrix(prix_global)

                
                self.clt_tache_prime(
                    tache_or_prime,
                    ligne.getObjectFromAttr('affaire'),
                    obj_salarie,
                    periode_id,
                    prix_global,
                    
                    )
                
                self.portal_workflow.doActionFor(ligne,'mouvementer')
        self.portal_workflow.doActionFor(journal,'mouvementer')


        

    
        
        
    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'clt_tache_prime')    
    def clt_tache_prime(self,tache_or_prime,affaire,salarie,periode_id,prix):
        """cloture une ligne du journal
        1 - affectation du prix au fichier du mouvement
        2 - si prime , maj du solde de la prime
        3 - si tache maj du salaire 
        """
        
        
        mvt_object=get_mvt_object(affaire,tache_or_prime)

        log("****************** PHASE 1 Debit tache ************************")

        ### maj de l'objet mouvement
        addtoattr(mvt_object,'prix_mois_%s' % periode_id,prix)
        addtoattr(mvt_object,'prix_exercice',prix)

        ### maj de l'objet prime
        #addtoattr(tache_or_prime,'prix_%s_mois_%s' % (periode_id,periode_id),prix)
        #addtoattr(tache_or_prime,'prix_%s_exercice' % (periode_id),prix)
        
        log("****************** PHASE 2 Maj fiche salarie ************************")
        #### maj de l'objet article
        if tache_or_prime.isPrime():
            ##### maj  de l'objet salarie pour les primes
            addtoattr(salarie,'prime_%s' % (periode_id,),prix)
            addtoattr(salarie,'prime_exercice' ,prix)
        else:
            ### maj du salaire global du salarie
            addtoattr(salarie,'salaire_%s' % (periode_id,),prix)
            addtoattr(salarie,'salaire_exercice' ,prix)
            
        log("****************** PHASE 3 Debit frais generaux ************************")
        
        if not isinstance(affaire,BatyFraisGeneraux):
            coefficient_generaux=tache_or_prime.getCoefficientValue()
            article=tache_or_prime
            article_frais_generaux = article.getObjectFromAttr('famille').getObjectFromAttr('coefficient').getObjectFromAttr('article_frais')
            affaire_frais_generaux = article.getEnt().getFirstObjectForType(type='BatyFraisGeneraux')
            if article_frais_generaux is None:
                raise BatyException(BatyException.BATY_JOURNAL_ERREUR_ARTICLE_FG)
            if affaire_frais_generaux is None:
                raise BatyException(BatyException.BATY_JOURNAL_ERREUR_AFFAIRE_FG)

            mvt_frais_generaux=get_mvt_object(affaire,article_frais_generaux)
            print "prix ===",prix,"  coefficient =",coefficient_generaux
            prix_total_frais=float(prix)*float(coefficient_generaux)/100.0
            
            addtoattr(mvt_frais_generaux,'prix_mois_%s' % periode_id,prix_total_frais)
            addtoattr(mvt_frais_generaux,'prix_exercice',prix_total_frais)

            ## maj article frais
            addtoattr(article_frais_generaux,'prix_entre_mois_%s' % periode_id,prix_total_frais)
            addtoattr(article_frais_generaux,'prix_entre_exercice' ,prix_total_frais)
            
            log("****************** PHASE 3 Credit frais generaux ************************")


            mvt_credit=get_mvt_object(affaire_frais_generaux,article_frais_generaux)
            log("credit",  affaire_frais_generaux.getId(),  mvt_credit.getId(),  prix_total_frais)         
            
            addtoattr(mvt_credit,'prix_mois_%s' % periode_id,prix_total_frais)
            addtoattr(mvt_credit,'prix_exercice',prix_total_frais)

            ### A voir ??? article credit = article frais generaux ???

            #addtoattr(credit,'prix_entre_mois_%s' % periode_id,prix_total_frais)
            #addtoattr(credit,'prix_entre_exercice',prix_total_frais)
            
            

    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'clt_periode' )
    def clt_periode(self,entreprise,Periode):
        """ cloture la priode et archive les resultats """

        log('debut cloture',Periode)
        periode_id=Periode.get_periode_type()
        log('debut cloture')
        ## verification que tous les journaux de la periode sont clotur
        portal_catalog=getToolByName(self,'portal_catalog')
        journaux=portal_catalog.search({'portal_type':['BatyJournal','BatyJournalHeure'],
                                        'state_mouvement':'creation',
                                       
                                        'path':'/'.join(entreprise.getPhysicalPath())})

      
        
        for j in journaux:
            
            if j.getObject().get_periode_type() == periode_id:
            
                raise BatyException(BatyException.BATY_ERREUR_CLOTURE_JOURNAL)
        ## a partir d'ici on est sur que tout les jaurnaux sont ferm


        
        ## on recupere tout les mouvements pour les cloturs
        brs=portal_catalog.search({'portal_type':['BatyMouvement','BatySalarie','BatyArticle','BatyPrime','BatyTache','BatyArticleCredit','BatyArticleFraisGeneraux','BatyArticleFraisStockage'],
                                   'path':'/'.join(entreprise.getPhysicalPath())})
        log('cloture step 2')
        for br in brs:
            
            mvt_object=br.getObject()
            
            mvt_object.clt_month(Periode)
        log('cloture step 3')
        ## a partir de ce moment on cloture la periode au niveau de l'entreprise
        entreprise.cloturePeriode(Periode.get_periode_type())

        ## pour la nouvelle periode on cree un journal analytique et un journal des heures
        entreprise.createNewJournal('BatyJournal',entreprise.getPeriodes()[0])
        entreprise.createNewJournal('BatyJournalHeure',entreprise.getPeriodes()[0])
        
        
        
            


    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'genere_rapport_journal_heure' )
    def genere_rapport_journal_heure(self,context):
        """ genere le rapport du journal """

        
        baty_rapport=BatyReport('rapport_journal',title= context.Title(),header_left= context.Title(),draw_line=1,landscape=1)

        baty_rapport.batyreport_header('Rapport : ' + context.Title(),)
        baty_rapport.batyreport_p("""<i>id:</i> <b>%s</b>""" % (context.getId(),))
        baty_rapport.batyreport_p("""<i>Periode:</i> <b>%s</b>""" % (context.getPeriode(),))
        baty_rapport.batyreport_p("""<i>Date de saisie:</i><b>%s</b>""" % (getattr(context,'date_de_saisie').strftime("%d/%m/%Y"),))
        baty_rapport.batyreport_p("""<i>Entreprise:</i> <b>%s</b>""" % (context.getEnt().Title(),))

        ####Priode Date cration Date de saisie
        #N Ligne N Document N Affaire Nom NArticle Nom 
        #Quantit Prix U Code frais indirect Mvt de Stock Mise a jour

        ### Rapport par affaire
        ### Rapport par salarie
        ### Rapport par tache


        data_header_affaire=['Affaire', 'Tache','Quantit', 'Prix Global']
        data_header_salarie=['Salarie', 'Tache','Quantit', 'Prix Global']
        data_header_tache=['Tache', 'Salarie','Quantit', 'Prix Global']
        
        
        list=context.objectItems('BatyJournalHeureLigne')
        list=[x[1] for x in list]
        list.sort(lambda x, y: cmp(x.getMetadataFromAttr('Affaire','Title'),y.getMetadataFromAttr('Affaire','Title'))  )
        
        i=0
        courante_date_saisie=''
        tup=[]
        for item in list:
            date_saisie=getattr(item,'creation_date')
            if not courante_date_saisie or courante_date_saisie.strftime('%Y%m%d') != date_saisie.strftime('%Y%m%d'):
                log("dans le if ",courante_date_saisie)
                courante_date_saisie=date_saisie
                if len(tup):
                    
                    baty_rapport.batyreport_table(tup)
                    baty_rapport.batyreport_page_break()
                    tup=[data_header]
                    baty_rapport.batyreport_header('Saisie du %02d/%02d/%04d' % (courante_date_saisie.day(),courante_date_saisie.month(),courante_date_saisie.year()))
                else:
                    ## c'est juste le debut
                    tup=[data_header]
                    baty_rapport.batyreport_header('Saisie du %02d/%02d/%04d' % (courante_date_saisie.day(),courante_date_saisie.month(),courante_date_saisie.year()))
            document=item.getDocument()

            log("on ajoute %d %s" % (i,archetype_tool.getObject(document)))
            article=archetype_tool.getObject(item.getArticle())
            coefficient=article.coefficient_value()
            l_item=[i,item.getDocument() and archetype_tool.getObject(item.getDocument()) or '-',archetype_tool.getObject(item.getAffaire()).Title(),archetype_tool.getObject(item.getArticle()).Title(),item.getQuantite(),item.getPrix_unitaire(),coefficient,'oui' and item.getStock() and int(item.getStock()) or 'non']
            tup.append(l_item)
            i=i+1
        if len(tup):
            ## il faut vider la table
            baty_rapport.batyreport_table(tup)
            
           
        
        



        
        context._setObject('rapport_journal',baty_rapport)



    

    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'create_affaire_from_devis' )
    def create_affaire_from_devis(self,entreprise,client_uid,name,list_mvt=[]):
        """ create an affair with the list of article pas in arguments """
        
        
        base_affaire=entreprise.getDestination('BatyAffaire')
        id=generate_id(prefix='BatyAffaire')
        base_affaire.invokeFactory(id=id,type_name='BatyAffaire')
        #base_affaire[id].setTitle(name)
        #base_affaire[id].setClient(client_uid)
        
        base_affaire[id].update(**{'title':name,'client':client_uid})
        affaire=base_affaire[id]

        ## on reindex
        #affaire.reindex()
        self._mvt_devis(affaire,list_mvt)
        return affaire
        
        

    def _mvt_devis(self,affaire,list_mvt=[]):
        """ create all baty mouvement in affair """
        i=0
        for item_from_cata in list_mvt:
            obj=item_from_cata.getObject()
            
            article_obj=self.getObjFromUid(obj.getArticle())
            frais_obj=article_obj.getArticleFraisObject()
            credit_obj=article_obj.getArticleCreditObject()
            frais_generaux_affaire=article_obj.getEntreprise().getFraisGenerauxAffaire()

            mvt_object=get_mvt_object(affaire,article_obj)
            mvt_frais_generaux=get_mvt_object(affaire,frais_obj)
            mvt_credit_frais_generaux=get_mvt_object(frais_generaux_affaire,credit_obj)
            
            
            
            
            real_quantite=obj.getRealQuantite()
            prix_achat=obj.getPrixAchat()*real_quantite
            
            part_frais_generaux=obj.getPrix()*real_quantite - prix_achat
            
            

            
            
            ## prix prevu sur l'article
            addtoattr(mvt_object,'prix_prevu',prix_achat)
            addtoattr(mvt_object,'quantite_prevu',real_quantite)
            ## ajoute au prix de vente de l'affaire
            addtoattr(affaire,'prix_vente',obj.getPrixVente()*real_quantite)

            
            ## prix prevu sur les frais generaux
            addtoattr(mvt_frais_generaux,'prix_prevu',part_frais_generaux)
            ## credit prevu sur crtedit frais generaus
            addtoattr(mvt_credit_frais_generaux,'prix_prevu',part_frais_generaux)
            

            
            
            
        
        

    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'add_devis_to_affaire' )
    def add_devis_to_affaire(self,affaire_uid,list_mvt=[]):
        """ add elements to an affair with the list of article"""
        affaire=self.getObjFromUid(affaire_uid)
        self._mvt_devis(affaire,list_mvt)
        return affaire
            
    security.declareProtected(BatyPermissions.BATY_MOUVEMENT,'update_situation' )
    def update_situation(self,devis,hash_situation={}):
        """ update the devis situation workflow variable"""
        portal_workflow=getToolByName(self,'portal_workflow')
        portal_workflow.doActionFor(devis,'situer',situation=hash_situation)
    

InitializeClass(BatyTool)    
