from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import Item
from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager
from Globals import InitializeClass, Persistent
from Acquisition import Implicit
from Products.Archetypes.public import DisplayList

def manage_addDefault_TVA(dispatcher):
    """ add an propertys default tva """
    p=Default_TVA()
    dispatcher.Destination()._setObject('tva',p)

class BatyProperty(Implicit, Persistent, PropertyManager, RoleManager, Item):

    meta_type='BatyProperty'
    
    def display(self):
        """ return an display list of the batyProperty """
        return DisplayList([ (x,self.getProperty(x)) for x in self.propertyIds() ])  


class BatyPropertyTVA(BatyProperty):

    meta_type='BatyProperty Default_TVA'
    security=ClassSecurityInfo()
    
    _properties=({'id':'tva19','type': 'float', 'mode':'wd'},
                 {'id':'tva5','type': 'float', 'mode':'wd'},
                 {'id':'tva0','type': 'float', 'mode':'wd'},
                 )

    def __init__(self):
        
        self.tva19=19.6
        self.tva5=5.5
        self.tva0=0.0

        

InitializeClass(BatyPropertyTVA)


class BatyPropertyCoefStruct(BatyProperty):

    meta_type='BatyProperty Coefficient structure'
    security=ClassSecurityInfo()
    
    _properties=({'id':'cs_2','type': 'float', 'mode':'wd'},
                 {'id':'cs_5','type': 'float', 'mode':'wd'},
                 {'id':'cs_10','type': 'float', 'mode':'wd'},
                 )

    

    def __init__(self):
        
        self.cs_2=2.0
        self.cs_5=5.0
        self.cs_10=10.0

InitializeClass(BatyPropertyCoefStruct)



class BatyPropertyCoefStockage(BatyProperty):

    meta_type='BatyProperty Coefficient frais stockage'
    security=ClassSecurityInfo()
    
    _properties=({'id':'cf_0','type': 'float', 'mode':'wd'},
                 {'id':'cf_5','type': 'float', 'mode':'wd'},

                 )

    

    def __init__(self):
        
        self.cf_0=0.0
        self.cf_5=5.0
        

InitializeClass(BatyPropertyCoefStockage)



class BatyPropertyCompteExploitation (BatyProperty):

    meta_type='BatyProperty Compte_exploitation'
    security=ClassSecurityInfo()
    
    _properties=({'id':'ce1','type': 'string', 'mode':'w'},
                 {'id':'ce2','type': 'string', 'mode':'w'},
                 {'id':'ce3','type': 'string', 'mode':'w'},
                 {'id':'ce4','type': 'string', 'mode':'w'},
                 )

    

    def __init__(self):
        
        setattr(self,'ce1','Debit')
        setattr(self,'ce2','Frais Indirects')
        setattr(self,'ce3','Crdit')
        setattr(self,'ce4','Rsultat')

InitializeClass(BatyPropertyCompteExploitation)




class BatyPropertyUnite(BatyProperty):

    meta_type='BatyProperty Unite'
    security=ClassSecurityInfo()
    
    _properties=({'id':'uni','type': 'string', 'mode':'w'},
	         {'id':'lin','type': 'string', 'mode':'w'},
                 {'id':'me2','type': 'string', 'mode':'w'},
                 {'id':'me3','type': 'string', 'mode':'w'},
                 {'id':'lit','type': 'string', 'mode':'w'},
                 {'id':'heu','type': 'string', 'mode':'w'},
                 {'id':'jou','type': 'string', 'mode':'w'},
                 {'id':'moi','type': 'string', 'mode':'w'},
                 {'id':'kil','type': 'string', 'mode':'w'},
                 {'id':'ton','type': 'string', 'mode':'w',}
                 )

    def __init__(self):
        
        self.lin='Linaire'
        self.me2='Metre 2'
        self.me3='Metre 3'
        self.lit='Litre'
        self.heu='Heure'
        self.jou='Jour'
        self.moi='Mois'
        self.kil='Killogramme'
        self.ton='Tonne'
	self.uni='Unite'

InitializeClass(BatyPropertyUnite)

class BatyPropertySecteur(BatyProperty):

    meta_type='BatyProperty Secteur'
    security=ClassSecurityInfo()
    
    _properties=({'id':'smat','type': 'string', 'mode':'w'},
                 {'id':'spro','type': 'string', 'mode':'w'},
                 {'id':'sfra','type': 'string', 'mode':'w'},
                 {'id':'sstk','type': 'string', 'mode':'w'},
                 )

    def __init__(self):
        
        self.smat='Matriel'
        self.spro='Produit'
        self.sfra='Frais gnraux'
        self.sstk='stock'

InitializeClass(BatyPropertySecteur)


class BatyPropertyFamille(BatyProperty):

    meta_type='BatyProperty Famille'
    security=ClassSecurityInfo()
    
    _properties=({'id':'mdo','type': 'string', 'mode':'w'},
                 {'id':'mat','type': 'string', 'mode':'w'},
                 {'id':'pro','type': 'string', 'mode':'w'},
                 {'id':'frg','type': 'string', 'mode':'w'},
                 {'id':'crd','type': 'string', 'mode':'w'},
                 )

    def __init__(self):
        
        self.mdo='Main d\'oeuvre'
        self.mat='Materiaux'
        self.pro='Produits'
        self.frg='Frais gnraux'
        self.crd='Credit'

InitializeClass(BatyPropertyFamille)




class BatyPropertyAffaire(BatyProperty):

    meta_type='BatyProperty Affaire'
    security=ClassSecurityInfo()
    
    _properties=({'id':'af1','type': 'string', 'mode':'w'},
                 {'id':'af2','type': 'string', 'mode':'w'},
                 {'id':'af3','type': 'string', 'mode':'w'},
                 {'id':'af4','type': 'string', 'mode':'w'},
                 {'id':'af5','type': 'string', 'mode':'w'},
                 {'id':'af6','type': 'string', 'mode':'w'},
                 {'id':'af7','type': 'string', 'mode':'w'},
                 )

    def __init__(self):
        
        self.af1='Main d\'oeuvre'
        self.af2='Article'
        self.af3='Produits demis finis'
        self.af4='Matriel'
        self.af5='Frais indirects'
        self.af6='Crdits'
        self.af7='Rsultats'

InitializeClass(BatyPropertyAffaire)


class BatyDefaultProperty(BatyProperty):

    meta_type='BatyDefaultProperty'
    security=ClassSecurityInfo()
    
    _properties=({'id':'heure_salarie_default','type': 'float', 'mode':'wd'},
                 {'id':'money_symbol','type': 'string', 'mode':'wd'},
                 )

    def __init__(self):
        
        self.heure_salarie_default=8.0
        self.money_symbol='&euro;'
        

InitializeClass(BatyDefaultProperty)


class BatyPropertyPlanComptable(BatyProperty):

    meta_type='BatyProperty Plan comptable'
    security=ClassSecurityInfo()
    
    _properties=({'id':'1','type': 'string', 'mode':'w'},
                 {'id':'2','type': 'string', 'mode':'w'},
                 {'id':'3','type': 'string', 'mode':'w'},
                 {'id':'4','type': 'string', 'mode':'w'},
                 {'id':'5','type': 'string', 'mode':'w'},
                 {'id':'6','type': 'string', 'mode':'w'},
                 {'id':'7','type': 'string', 'mode':'w'},
                 )

    def __init__(self):
        setattr(self,'1','Actifs')
        setattr(self,'2','Passif')
        setattr(self,'3','Chifre d\'affaires r\xE9sultant des ventes et des prestations de services')
        setattr(self,'4','Charges de mati\xE8re et de marchandises')
        setattr(self,'5','Charges de personnel')
        setattr(self,'6','Autres charges d\'exploitation')
        setattr(self,'7','R\xE9sultat des activit\xE9s annexes d\'exploitation')
        setattr(self,'8','R\xE9sultats exceptionnels et hors exploitation')
        setattr(self,'9','Cl\xF4ture')


        
InitializeClass(BatyPropertyPlanComptable)
